#include <SoftwareSerial.h>
#include <avr/sleep.h>

#define SENSORPIN	(0)
#define WAKEPIN		(8)
#define SENSORINPUTPIN	(13)

const char szUUIDChara1[] = "022EA09A348A11E8B4670ED5F89F718B";

void setup()
{
	// スリープ中の消費電力を最小限に抑えるために、
	//  i) 出力ピンは電流を供給なし
	// ii) 入力ピンは一定
	// にする。
	// i) ii) を実現する簡単な方法は、内部プルアップ抵抗を有効にすること。
	for( int i = 0; i < 20; i++ )
	{
		pinMode(i, INPUT_PULLUP);
	}

	// センサーピンのセットアップ
	// （省電力化のためにすべてのピンの内部プルアップ抵抗を有効にした。
	// 　プルアップ抵抗を有効にしたままanalogRead()でアナログ値を読み取ると、
	// 　正しい値が返ってこない）
	// アナログピンの場合、pinMode()の引数のピンの番号は、(14+アナログピン番号)を指定する。
	pinMode( 14 + SENSORPIN, INPUT );

	// WAKEピンのセットアップ
	pinMode( WAKEPIN, OUTPUT );
	digitalWrite( WAKEPIN, HIGH );

	// センサーインプットピンのセットアップ
	pinMode( SENSORINPUTPIN, OUTPUT );
	digitalWrite( SENSORINPUTPIN, LOW );

	// RN4020の起動待ちウェイト
	delay( 1000 );	// 1秒

	// RN4020をスリープさせる
	digitalWrite( WAKEPIN, LOW );
}

void loop()
{
	// ATmega328自身をスリープさせる
	Sleep();

	// （外部割り込みでATmega328自身はスリープから復帰済み）

	// RN4020をスリープから復帰させる
	digitalWrite( WAKEPIN, HIGH );

	// 測定と送信
	ReadAndSend();

	// RN4020をスリープさせる
	digitalWrite( WAKEPIN, LOW );
}

// 測定と送信
void ReadAndSend()
{
	// ボーレートを指定して通信開始
	SoftwareSerial rn4020( 6, 7 );
	rn4020.begin( 115200 );
	delay( 100 );	// 0.1秒：安定化待ち

	// センサーに通電
	digitalWrite( SENSORINPUTPIN, HIGH );

	// 測定
	int iValue = analogRead( SENSORPIN );

	// センサーの通電の解除
	digitalWrite( SENSORINPUTPIN, LOW );

	// 測定値のBLEモジュールへの書き込み
	char szCommand[1024] = "";	// 送信コマンド
	sprintf( szCommand, "SUW,%s,%04X", szUUIDChara1, iValue );	// コマンド作成
	rn4020.println( szCommand );	// コマンド送信
}

void wakeUpNow()
{
	; // 何もなし
}

// ATmega328自身のスリープ
void Sleep()
{
	// ① アナログ・デジタル・コンバータ機能を無効にする（少しでも電力消費を抑えるため）
	byte adcsra_old = ADCSRA;	// ADC Control と Register Aの設定値の以前の値を保持。
	ADCSRA = 0;		// アナログ・デジタル・コンバータ機能の無効にする。

	// ② スリープモードを設定する（②⑤⑧でセット）
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);

	// ③ 割り込み機能を無効にする（④⑤の処理を確実に実行するため）
	noInterrupts();	// cli()でも同じ

	// ④ 電圧降下検出機能を無効にする（少しでも電力消費を抑えるため）
	sleep_bod_disable();

	// ⑤ スリープ機能を有効にする（②⑤⑧でセット）
	sleep_enable();

	// ⑥ 割り込み機能を有効にする（④⑤の処理を実行したので元に戻す）
	interrupts();	// sei()でも同じ

	// ⑦ 外部割り込み条件を設定する（Wakeするための外部割り込み条件の割込設定）
	attachInterrupt(0, wakeUpNow, HIGH);	// INT0端子(２番ピン)の立ち上がりで割り込み発生。wakeUpNow関数が呼ばれる。

	// ⑧ スリープ実行（②⑤⑧でセット）
	sleep_cpu();

	// （⑧によってスリープに移行する。⑦で設定した外部割り込みでWakeし、⑨の処理が実施される）

	// ⑨ スリープ機能を無効にする（⑦で設定した外部割り込みでWakeしたので、スリープを無効にする）
	sleep_disable();

	// ⑩ 外部割り込み条件の設定を解除する（Wakeしたので）
	detachInterrupt(0);

	// ⑪ アナログ・デジタル・コンバータ機能を有効にする（Wakeしたので）
	ADCSRA = adcsra_old;
}
